/*
This lib is a bind of vtkSufaceBooleanOperations for python.
To compile this library you gotta have gts e glib and vtkSufaceBooleanOperations 
installed on you computer

Bruno Lara Bottazzini - CTI 2011
*/
#include <vtkSurfaceBooleanOperations.h>
#include <vtkPolyData.h>
#include <vtkSphereSource.h>
#include <vtkXMLPolyDataWriter.h>
#include <vtkXMLPolyDataReader.h>
#include <vtkSTLReader.h>
#include <vtkSmartPointer.h>
#include <string.h>


#include "BooleanOperations.h"

//Here it is somethings that you have to link in Windows
#ifdef WIN32
#pragma comment(lib, "vtkSurfaceBooleanOperations.lib")
#pragma comment(lib, "gts.lib")
#pragma comment(lib, "glib-2.0.lib")
#endif

#define vtkNew(type,name) \
  vtkSmartPointer<type> name = vtkSmartPointer<type>::New()

/*
Method: SetOperationType
You set the type of the BooleanOperation here you can set
Union, Difference and Intersection. Then you have to pass
through as a string.
*/	
void BooleanOperations::SetOperationType(char type[]){
	long tam = strlen(type);
	//USE: CALLOC FOR CHARACTERS
	this->type = (char *) calloc(tam,sizeof(type));
	strcpy(this->type,type);
}
/*
Method: SetFileName1
You set the path to the file1. File 1 is the file that
you'd like to make boolean operation
*/
void BooleanOperations::SetFileName1(char name[]){
	long tam = strlen(name);
	//USE: CALLOC FOR CHARACTERS
	this->name = (char *) calloc(tam,sizeof(name));
	strcpy(this->name,name);
}

/*
Method: SetFileName2
You set the path to the file1. File 2 is the file that
you'd like to make boolean operation
*/
void BooleanOperations::SetFileName2(char name2[]){    
	long tam = strlen(name2);
	//USE: CALLOC FOR CHARACTERS
	this->name2 = (char *) calloc(tam,sizeof(name2));
	strcpy(this->name2,name2);
}

/*
Method: SetFileOutput
The path of the Output file.
*/
void BooleanOperations::SetFileNameOutput(char output[]){    
	long tam = strlen(output);
	//USE: CALLOC FOR CHARACTERS
	this->output = (char *) calloc(tam,sizeof(output));
	strcpy(this->output,output);
}

/*
Method: Update
When you finished to set everything then you type the update.
It's when it starts to make the booleanOperation
*/
bool BooleanOperations::Update(){

  int i = 0;

  if(this->name == false)return 0;
  if(this->name2 == false)return 0;
  if(this->output == false)return 0;
  if(this->type == false)return 0;
  this->SetProgress(0.0);

  vtkNew(vtkSurfaceBooleanOperations,booleanOperator);  
  vtkNew(vtkXMLPolyDataReader, r1);
  vtkNew(vtkXMLPolyDataReader, r2);

  this->SetProgress(0.1);
  r1->SetFileName(this->name);
  r1->Update();
  r2->SetFileName(this->name2);
  r2->Update();
  this->SetProgress(0.2);
  booleanOperator->AddInputConnection(r1->GetOutputPort());
  this->SetProgress(0.4);
  booleanOperator->AddInputConnection(r2->GetOutputPort());
  this->SetProgress(0.6);
  vtkNew(vtkXMLPolyDataWriter,writer);
  writer->SetInputConnection(booleanOperator->GetOutputPort());

  //Upper case the string
  for(i = 0 ; i < strlen(this->type) ; i++){
    this->type[i] = toupper(this->type[i]);
  }
  this->SetProgress(0.7);
  if(!(strcmp("UNION",this->type))){
	booleanOperator->SetModeToUnion();
  }else{
		if(!(strcmp("DIFFERENCE",this->type))){
			 booleanOperator->SetModeToDifference();
		}else{
			if(!(strcmp("INTERSECTION",this->type))){
				booleanOperator->SetModeToIntersection();
			}else{
				return 0;	
			}
		}
  }

  this->SetProgress(0.9);
  writer->SetFileName(this->output);
  writer->Update();
  this->SetProgress(1.0);
  return 1;

}

/*
Method: GetProgress()
This method return the progress that the boolean Operation is.
*/
double BooleanOperations::GetProgress(){
return this->progress;
}

/*
Method: SetProgress()
This private method the class set it-self the progress
you just have to get it with GetProgress
*/
void BooleanOperations::SetProgress(double progress){
	this->progress = progress;
}
